# This function returns the point x.r in [x.min,x.max]
# at which exp(-llik(x)+llik(x.opt))=r
# In statistical terms, llik is the MINUS log lik function, x.opt the MLE,
# SIDE indicates on what side of x.opt one is going to look for x.r.

# [x.min,x.max] an interval of values of x where one is likely to
# find x.r where the normed likelihood is r: containing x.r 
# is only necessary for the SECANT method.
# x.min is the only required value (i.e. x.max is useless) when one
# uses the STEFFENSEN method. But x.min must be in a neighbourhood
# of x.opt. This last method converges more quickly, but its is more
# sensitive to the initial condition.

# Note that x.opt is an indicating value. It first precisely computed
# using nlm (unless opt is set to F)
# eps is the precision required in the determination of x.r

# The secant method to determine the roots of an equation: 
# here, R=0.10 for example

lik.border <- function(llik,x.opt,x.min,x.max,r=0.10,eps=0.0001,
               opt=T,method="dicho",side,output=F){
if (opt){
 tamp <- nlm(llik,p=x.opt)
 x.opt <- tamp$estimate
 if (side=="left") x.max <- min(0.99*x.opt,x.max)
 else x.min <- max(1.01*x.opt,x.min)}
#if (output) cat("xmin and xmax: ",x.min,x.max,"\n")
lr <- log(r)
ll.opt <- llik(x.opt)

x.old <- x.min
x.new <- x.max

r.old <- r.min <- -llik(x.old)+ll.opt
r.new <- r.max <- -llik(x.new)+ll.opt

if (is.na(r.min)) {
  cat("llik is NA at x.min=",x.min,"\n")
  print(" ==> choose a larger value for the lower bound?")
  return(list(x.opt=x.opt,x.r=NA,r=NA))}
if (is.na(r.max)) {
  cat("llik is NA at x.max=",x.max,"\n")
  print(" ==> choose a smaller value for the upper bound?")
  return(list(x.opt=x.opt,x.r=NA,r=NA))}

dx <- x.max-x.min
rt <- switch(method,
 secant={
  if (dx==0){
   cat("x.min and x.max must be different","\n")
   return(1)}
  while (abs(r.new-lr)>eps){
   x.new <- x.min-(r.min-lr)*(dx/(r.max-r.min))
   r.new <- -llik(x.new)+ll.opt
   if (r.new>lr) {
    if (side=="lefat"){
     x.max <- x.new
     r.max <- r.new}
    else {
     x.min <- x.new
     r.min <- r.new}}
   else {
    if (side=="left"){
     x.min <- x.new
     r.min <- r.new}
    else {
    x.max <- x.new
    r.max <- r.new}}
   dx <- x.max-x.min
#   cat(x.new,exp(r.new),x.min,x.max,dx,"\n")
 }},
 Steffensen={
  x.new <- x.old
  r.new <- r.old
  cat("starting value:", x.new,"\n")
  while (abs(r.new-lr)>eps){
    tamp <- x.new-((r.new-lr)^2/(-llik(x.new+r.new-lr)+ll.opt-r.new))
    x.new <- tamp
    cat("Xnew: ",x.new,"\n")
    r.new <- -llik(x.new)+ll.opt}},
 dicho={
  if ((r.min-lr)*(r.max-lr)>0) {
    cat("[x.min;x.max] does not contain the required R value","\n")
#    cat("R at x.min",exp(r.min),"\n")
#    cat("R at x.max",exp(r.max),"\n")
    return(2)}
  while(dx>eps) {
   x.new <- 0.5*(x.min+x.max)
   r.new <- -llik(x.new)+ll.opt
#   cat("(",x.min,x.new,x.max,")",exp(r.new),"\n")
   if (r.new>lr) {
    if (side=="left")
     x.max <- x.new
    else x.min <- x.new}
   else {
    if (side=="left")
     x.min <- x.new
    else x.max <- x.new}
   dx <- x.max-x.min}})
if (is.null(rt)){
 print("The method specified in LIK.BORDER must be either")
 print("secant, Steffensen or dicho.")
 return(3)}
return(list(x.opt=x.opt,x.r=x.new,r=exp(r.new)))}

